#####################################################
#
#	R code for setting-up fixed-effects meta-analysis
#	of LDL data for Atorvastatin 10 mg
#
#####################################################

#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R
library(R2WinBUGS)
#library(lattice)  # For using trellis graphics
#library(MASS)

#--- Set working directory ---#
setwd("DIRECTORY WHERE YOU'RE KEEPING YOUR FILES")

#--------- Question 1 ---------#
#--- Read and print the data ---#

d <- read.csv('AtorvastatinStudies.csv')
head(d)

#--------- Question 2 ---------#
#--- Calculate difference from placebo and its standard errors ---#

#--- Subset to keep only the atorvastatin groups ---#
ator <- d[d$dose1>0,]

#--- Subset to keep only the placebo groups ---#
pbo <- d[d$dose1==0,c("trial","n","ldlPcfb","ldl0",
		"seLdl0","seLdlPcfb")]
names(pbo) <- paste(names(pbo),".pbo",sep='')

#--- Merge to make one dataset ---#
ator <- merge(ator, pbo, by.x="trial",by.y="trial.pbo", all=T)

#--- Calculate the difference and associated standard error ---#
ator$diff <- ator$ldlPcfb - ator$ldlPcfb.pbo
ator$seDiff <- sqrt(ator$seLdlPcfb^2 + ator$seLdlPcfb.pbo^2)

#--------- Question 3 ---------#
#-- Make some forest plots of the data --#

ator80 <- ator[ator$dose1==80,]

# Simplest plot
forest(x=ator80$diff, sei=ator80$seDiff)

# Add a few key things
forest(x=ator80$diff, sei=ator80$seDiff, 
	slab=ator80$trial, refline=NULL, 
	xlab="Mean Difference",
	main="Difference in %CFB LDL for 80 mg Atorvastatin")

# More control of order, labels, axes, etc.
head(ator80)
ator80 <- ator80[rev(order(ator80$diff)),]
head(ator80)

forest(x=ator80$diff, sei=ator80$seDiff, 
	slab=ator80$trial, refline=NULL, 
	at=seq(-80,-30,by=10),ylim=c(0,9),
	xlab="Mean Difference",
	main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-120.5, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-35, 7.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)

#--------- Question 4 ---------#

#-- Fit fixed effects model by-hand --#
ator80$w <- 1/ator80$seDiff^2
print(ator80$w/sum(ator80$w))

feEst <- sum(ator80$diff * ator80$w) / sum(ator80$w)
seFeEst <- sqrt( 1/sum(ator80$w) )
feCI <- feEst + c(-1.96, 0, 1.96)*seFeEst
names(feCI) <- c("LCL95","Estimate","UCL95")
print(feCI)


#--------- Question 5 ---------#

#-- Make forest plot with FE estimate added ---#
forest(x=ator80$diff, sei=ator80$seDiff, 
	slab=ator80$trial, refline=NULL, 
	at=seq(-80,-30,by=10),ylim=c(-2.5,nrow(ator80)+3),
	xlab="Mean Difference",
	main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-120.5, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)
addpoly(x=feEst, sei=seFeEst, mlab="Fixed Effects Estimate", row=-1)

#--------- Question 6 ---------#

#--- Fit model using the rma funciton
fema <- rma(yi=diff, sei=seDiff, data=ator80, method="FE", slab=trial)

print(fema)
plot(fema)
forest(fema)

forest(fema, refline=NULL, 
	at=seq(-80,-30,by=10),
	ylim=c(-2.5,nrow(ator80)+3),
	xlab="Mean Difference",
	main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-120.5, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)


#--------- Question 7 ---------#






