#####################################################
#
#	Introduction to metafor
#
#####################################################

#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R
library(R2WinBUGS)
library(emdbook)


#--- Set working directory ---#
#setwd("DIRECTORY WHERE YOU'RE KEEPING YOUR FILES")
setwd("E:/MI260USB/MI260/FixedEffectsHandsOn")
#--- Read the data ---#

d <- read.csv('AtorvastatinStudies.csv')
head(d)

#--- Calculate difference from placebo and its standard error ---#

#--- Subset to keep only the atorvastatin groups ---#
ator <- d[d$dose1>0,]

#--- Subset to keep only the placebo groups ---#
pbo <- d[d$dose1==0,c("trial","n","ldlPcfb","ldl0",
		"seLdl0","seLdlPcfb")]
names(pbo) <- paste(names(pbo),".pbo",sep='')

#--- Merge to make one dataset ---#
ator <- merge(ator, pbo, by.x="trial",by.y="trial.pbo", all=T)

#--- Calculate the difference and associated standard error ---#
ator$diff <- ator$ldlPcfb - ator$ldlPcfb.pbo
ator$seDiff <- sqrt(ator$seLdlPcfb^2 + ator$seLdlPcfb.pbo^2)

#-- Make some forest plots of the data --#

ator10 <- ator[ator$dose1==10,]

# Table of data (compare to slides)
ator10[order(ator10$diff),c('trial','diff','seDiff')]


#--- Forest plots ---#
# We'll use the forest function to make these plots
# ... see help file

# Simplest plot
forest(x=ator10$diff, sei=ator10$seDiff)

# Add a few key things - study labels, remove reference line, add title
forest(x=ator10$diff, sei=ator10$seDiff, 
	slab=ator10$trial, refline=NULL, 
	xlab="Mean Difference",
	main="Difference in %CFB LDL for 10 mg Atorvastatin")

# More control of order, labels, axes, etc.
head(ator10)
ator10 <- ator10[rev(order(ator10$diff)),]
head(ator10)

with(ator10,
     forest(x=diff, sei=seDiff, 
            slab=trial, refline=NA, 
            at=seq(-80,-10,by=10),
            ylim=c(0,11),
#            xlim=c(-120,40),
            xlab="Mean Difference",
            main="Difference in %CFB LDL for 10 mg Atorvastatin")
)
text(-90.5, 9.5, "Study", cex=0.95, adj=0, font=2)
text(-15, 9.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)

#--- Fit model using the rma function
fema <- rma(yi=diff, sei=seDiff, data=ator10, method="FE", slab=trial)
fema2 <- rma(m1i=ldlPcfb, m2i=ldlPcfb.pbo, 
             sd1i=sqrt(n)*seLdlPcfb, sd2i=sqrt(n.pbo)*seLdlPcfb.pbo,
             n1i=n, n2i=n.pbo,
             measure="MD",
             data=ator10, method="FE", slab=trial)
print(fema)
plot(fema)
forest(fema)

forest(fema, refline=NULL, 
       at=seq(-80,-10,by=10),
       ylim=c(-2.5,nrow(ator10)+3),  
       xlab="Mean Difference",
       main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-90.5, nrow(ator10)+1.5, "Study", cex=0.95, adj=0, font=2)
text(-15, nrow(ator10)+1.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)
# Adding additional estimates
addpoly(fema2, row=-2, col='red', mlab='Alternative function call')


#--- Fitting FE model usign WinBUGS ---#
# Define data for BUGS fit
bugsdata <- with(ator10, 
                 list(thetahat=diff, s=seDiff, 
                      nStudies=length(trial))
                 )

bugsParameters <- c("theta")

bugsInits <- function() {
  return(list(theta=rnorm(1, -20, 20))) 
}

feBugs <- bugs(data=bugsdata, inits=bugsInits, 
               parameters.to.save=bugsParameters,
               model='feModel.bug',
               n.chains=3, n.iter=10000, n.burnin=2000, n.thin=1,
               debug=TRUE)

# look at convergence diagnostics
plot(feBugs)
print(feBugs)
# convert bugs output to coda format
feBugscoda <- as.mcmc.bugs(feBugs)
plot(feBugscoda)

# Add Bayes estimate to plot
forest(fema, refline=NULL, 
       at=seq(-80,-10,by=10),
       ylim=c(-2.5,nrow(ator10)+3),  
       xlab="Mean Difference",
       main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-90.5, nrow(ator10)+1.5, "Study", cex=0.95, adj=0, font=2)
text(-15, nrow(ator10)+1.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)
# Adding additional estimates
addpoly(x=mean(feBugs$sims.list$theta), 
        vi=var(feBugs$sims.list$theta),
        row=-2, col='red', mlab='Bayesian fit')


# Random effects model
rema <- rma(yi=diff, sei=seDiff, data=ator10, method="REML", slab=trial)

bugsParameters <- c("thetaMean","sdThetaMean","theta")

bugsInits <- function() {
  return(list(thetaMean=rnorm(1, -20, 20), sdThetaMean=runif(1,0,5),
		theta=rep(0,bugsdata$nStudies))) 
}

reBugs <- bugs(data=bugsdata, inits=bugsInits, 
               parameters.to.save=bugsParameters,
               model='reModel.bug',
               n.chains=3, n.iter=10000, n.burnin=2000, n.thin=1)

# Add Bayes estimate to plot
forest(rema, refline=NULL, 
       at=seq(-80,-10,by=10),
       ylim=c(-2.5,nrow(ator10)+3),  
       xlab="Mean Difference",
       main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-90.5, nrow(ator10)+1.5, "Study", cex=0.95, adj=0, font=2)
text(-15, nrow(ator10)+1.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)
# Adding additional estimates
addpoly(x=mean(reBugs$sims.list$thetaMean), 
        vi=var(reBugs$sims.list$thetaMean),
        row=-2, col='red', mlab='Bayesian fit')

