
#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R
library(R2WinBUGS)
library(emdbook)

# Set my working directory
setwd("E:/MI260USB/MI260/FixedEffectsHandsOn")

# Read the data
d <- read.csv('CochranePancreaticData12months.csv')
d$logOR <- with(d, log(d.GC*(N.G-d.G)/(d.G*(N.GC-d.GC))))
d$varLogOR <- with(d, 1/d.GC + 1/(N.GC-d.GC) + 1/d.G + 1/(N.G-d.G))


#-- Fit the model using the first method (asymptotic variances for logOR)

bugsmodel1 <- function() {
  for (i in 1:nStudies) {
    thetahat[i] ~ dnorm(theta[i], precThetaHat[i])
    theta[i] ~ dnorm(meanDiff, prec)
    precThetaHat[i] <- pow(s[i],-2)
  }
  meanDiff ~ dnorm(0,1E-04)
  tau ~ dunif(0,50)
  prec <- pow(tau,-2)
}

bugsdata1 <- list(thetahat=d$logOR, s=sqrt(d$varLogOR), 
                      nStudies=length(d$study))

bugsParameters1 <- c("meanDiff","tau","theta")

bugsInits1 <- function() {
  return(list(meanDiff=rnorm(1, 0, 5), tau=runif(1,0,5))) 
}

reBugs1 <- bugs(data=bugsdata1, inits=bugsInits1, 
                parameters.to.save=bugsParameters1,
                model=bugsmodel1,
                n.chains=3, n.iter=50000, n.burnin=10000, n.thin=1,
                debug=TRUE)


#--- Now fit a model using Binomial distribution ---#

# This model estimates a separate fixed effect for each
# Gemcitabine arm (alpha). These are really nuisance parameters
# and not of particular interest for this comparision.
# The study-specific log odds ratios (thetas) are comparable
# to the same parameters from the first model.
# The primary inference is on meanDiff and tau, which
# can be directly compared between the two models.


bugsmodel2 <- function() {
  for (i in 1:nStudies) {
    # Normal distribution for study-specific log-odds ratios
	theta[i] ~ dnorm(meanDiff, prec)

    # Fixed effects for logit(p) in Gemcitabine groups.
    # These are nuisance parameters in our model.
	alpha[i] ~ dnorm(0.0, 1.0E-04)  

    # Define probabilities for G
	logit(pG[i]) <- alpha[i]
    # Define probabilities for GC
	logit(pGC[i]) <- alpha[i] + theta[i]
 
    # Model for the data
	d.G[i] ~ dbin(pG[i], N.G[i])
	d.GC[i] ~ dbin(pGC[i], N.GC[i])
  }

  # Prior distributions for mean logOR and inter-trial variance in logOR
  meanDiff ~ dnorm(0,1E-04)
  tau ~ dunif(0,50)
  prec <- pow(tau,-2)
}


bugsdata2 <- list(d.G=d$d.G, N.G=d$N.G, d.GC = d$d.GC, N.GC = d$N.GC,
                      nStudies=length(d$study))

bugsParameters2 <- c("meanDiff","tau","theta","alpha")

bugsInits2 <- function() {
  return(list(meanDiff=rnorm(1, 0, 5), tau=runif(1,0,5), 
			alpha=rnorm(bugsdata2$nStudies, 0, 1))) 
}

reBugs2 <- bugs(data=bugsdata2, inits=bugsInits2, 
                parameters.to.save=bugsParameters2,
                model=bugsmodel2,
                n.chains=3, n.iter=50000, n.burnin=10000, n.thin=1,
                debug=TRUE)

# Compare model results for meanDiff ...
summary(reBugs1$sims.list$meanDiff)
summary(reBugs2$sims.list$meanDiff)
plot(density(reBugs1$sims.list$meanDiff))
lines(density(reBugs2$sims.list$meanDiff),col='red')

# ... and tau
summary(reBugs1$sims.list$tau)
summary(reBugs2$sims.list$tau)
plot(density(reBugs1$sims.list$tau))
lines(density(reBugs2$sims.list$tau),col='red')


# Plot the data and model fits
with(d[order(d$logOR),], {
	fit.re <- rma(logOR, vi=varLogOR, slab=study)
	forest(fit.re,ylim=c(-3.5, nrow(d)+3), atransf=exp,
		 mlab="Classical RE Model")
}
)
addpoly(mean(reBugs1$sims.list$meanDiff),
	  vi=var(reBugs1$sims.list$meanDiff), atransf=exp,
	  col='red', row=-2, cex=1, mlab="Bayes RE Model 1")
addpoly(mean(reBugs2$sims.list$meanDiff),
	  vi=var(reBugs2$sims.list$meanDiff), atransf=exp,
	  col='green', row=-3, cex=1, mlab="Bayes RE Model 2")



