#####################################################
#
#	R code for exploring heterogeneity and fitting
#	random effects meta-analysis models for the
#	LDL data for Atorvastatin 80 mg
#
#####################################################

#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R
library(lattice)	# For making scatter plots

#--- Set working directory ---#
setwd("DIRECTORY WHERE YOU'RE KEEPING YOUR FILES")

#--------- Data preparation ---------#
d <- read.csv('AtorvastatinStudies.csv')
head(d)

#--- Subset to keep only the atorvastatin groups ---#
ator <- d[d$dose1>0,]

#--- Subset to keep only the placebo groups ---#
pbo <- d[d$dose1==0,c("trial","n","ldlPcfb","ldl0",
		"seLdl0","seLdlPcfb")]
names(pbo) <- paste(names(pbo),".pbo",sep='')

#--- Merge to make one dataset ---#
ator <- merge(ator, pbo, by.x="trial",by.y="trial.pbo", all=T)

#--- Calculate the difference and associated standard error ---#
ator$diff <- ator$ldlPcfb - ator$ldlPcfb.pbo
ator$seDiff <- sqrt(ator$seLdlPcfb^2 + ator$seLdlPcfb.pbo^2)

ator80 <- ator[ator$dose1==80,]

#--------- Part 1: Small study effects ---------#

# Fit a model to use for making the plots
fema <- rma(yi=diff, vi=seDiff^2, data=ator80, method="FE", slab=trial)

# Funnel plot
funnel(fema, xlab="Difference in %CFB LDL", ylab="Standard Error")

# Radial plot
radial(fema)

#--------- Part 2. Meta-regression ---------#

#--- Question 1 ---#

# Scatter plot: difference vs. baseline LDL
#with size of points relative to meta-analysis weight
xyplot(diff~ldl0, data=ator80, pch=19, cex=9/ator80$seDiff^2,
	xlab="Baseline LDL", ylab="Difference from placebo",
	sub="Size of points proportional to meta-analysis weight")

# Forest plot with baseline LDL added
ord <- order(ator80$ldl0)

forest(x=ator80$diff[ord], vi=ator80$seDiff[ord]^2, 
	slab=ator80$trial[ord], refline=NA,
	ilab=signif(ator80$ldl0[ord],3),
	ilab.xpos=-90,
	ylim=c(0,nrow(ator80)+3),
	main="Difference from Placebo in %CFB LDL")
text(-120, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-95, 7.75, "Baseline\nLDL", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean diff. + 95%CI",cex=0.95, adj=0, font=2)


# Scatter plot: difference vs. change from baseline in placebo group
xyplot(diff~ldlPcfb.pbo, data=ator80, pch=19, cex=9/ator80$seDiff^2,
	xlab="Percent change from baseline in placebo group", 
	ylab="Difference from placebo",
	sub="Size of points proportional to meta-analysis weight")

# Forest plot with baseline LDL added
ord <- order(ator80$ldlPcfb.pbo)

forest(x=ator80$diff[ord], vi=ator80$seDiff[ord]^2, 
	slab=ator80$trial[ord], refline=NA,
	ilab=signif(ator80$ldlPcfb.pbo[ord],2),
	ilab.xpos=-90,
	ylim=c(0,nrow(ator80)+3),
	main="Difference from Placebo in %CFB LDL")
text(-120, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-95, 7.75, "Placebo\n%CFB LDL", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean diff. + 95%CI",cex=0.95, adj=0, font=2)


#--- Question 2 ---#
ator80 <- ator80[order(ator80$ldlPcfb.pbo),]
feReg <- rma(yi=diff, sei=seDiff, mods=(ldlPcfb.pbo), data=ator80, 
	method="FE", slab=trial)

forest(feReg, refline=NA, #order='obs',
	ilab=signif(ator80$ldlPcfb.pbo,2),
	ilab.xpos=-90,
	ylim=c(0,nrow(ator80)+3),
	main="Difference from Placebo in %CFB LDL")
text(-120, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-95, 7.75, "Placebo\n%CFB LDL", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean diff. + 95%CI",cex=0.95, adj=0, font=2)

#--- Question 3 ---#

reReg <- rma(yi=diff, sei=seDiff, mods=(ldlPcfb.pbo), data=ator80, 
	method="REML", slab=trial)

predVals <- predict(reReg, newmods=c(-2,0,2,4))

forest(reReg, refline=NA, #order='obs',
	ilab=signif(ator80$ldlPcfb.pbo,2),
	ilab.xpos=-90,
	ylim=c(-4,nrow(ator80)+3),
	main="Difference from Placebo in %CFB LDL")
text(-120, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-95, 7.75, "Placebo\n%CFB LDL", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean diff. + 95%CI",cex=0.95, adj=0, font=2)
addpoly(x=predVals$pred, sei=predVals$se,col='red',  cex=0.9,
	mlab=paste("Placebo response =",c(-2,0,2,4),"%"))






