#####################################################
#
#	R code for exploring heterogeneity and fitting
#	random effects meta-analysis models for the
#	LDL data for Atorvastatin 80 mg
#
#####################################################

#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R

#--- Set working directory ---#
setwd("DIRECTORY WHERE YOU'RE KEEPING YOUR FILES")

#--------- Data preparation ---------#
d <- read.csv('AtorvastatinStudies.csv')
head(d)

#--- Subset to keep only the atorvastatin groups ---#
ator <- d[d$dose1>0,]

#--- Subset to keep only the placebo groups ---#
pbo <- d[d$dose1==0,c("trial","n","ldlPcfb","ldl0",
		"seLdl0","seLdlPcfb")]
names(pbo) <- paste(names(pbo),".pbo",sep='')

#--- Merge to make one dataset ---#
ator <- merge(ator, pbo, by.x="trial",by.y="trial.pbo", all=T)

#--- Calculate the difference and associated standard error ---#
ator$diff <- ator$ldlPcfb - ator$ldlPcfb.pbo
ator$seDiff <- sqrt(ator$seLdlPcfb^2 + ator$seLdlPcfb.pbo^2)

ator80 <- ator[ator$dose1==80,]

#--------- Question 1 ---------#
#- Calculate Q and test H0:no heterogeneity -#
w <- ator80$seDiff ^ -2 
feEst <- sum(w * ator80$diff) / sum(w)
seFeEst <- sqrt( 1/sum(w) )

k <- nrow(ator80)-1
Q <- sum( w*(ator80$diff - feEst)^2 )
pvalue.Q <- 1-pchisq(Q,k)

#--------- Question 2 ---------#
#- Calculate H and I2 -#
H <- sqrt(Q/k)
I2 <- 100*(Q-k)/Q

#--------- Question 3 ---------#
#- Calculate D-L estimate of between-study variance and RE est by hand -#
tau2.dl <- max( 0 , (Q - k) / ( sum(w) - sum(w^2)/sum(w) ) )
wre <- 1/(ator80$seDiff^2 + tau2.dl)
reEst <- sum(wre * ator80$diff) / sum(wre)
varReEst <- 1/sum(wre)
reCI <- reEst + c(-1.96, 0, 1.96)*sqrt(varReEst)

# Compare the relative weights between fixed and random effects 
# models
print(w / sum(w))
print(wre / sum(wre))

#- Plot relative weights for fixed- and random-effects models
plot(1:length(w), w/sum(w), type='b', bg='black', pch=21, xlab="Study",
	ylab="Meta-analysis Weights", ylim=c(0,.3))
points(1:length(wre), wre/sum(wre), type='b', col='red',bg='red', pch=21)
abline(h=1/length(w), col='grey', lty=2)
legend('bottomleft',legend=c("Fixed effects model","Random effects model"), lty=c(1,1),
	pch=c(21,21), pt.bg=c("black","red"),col=c('black','red'))


#--------- Question 4 ---------#

#- Fit RE (Dersimonian and Laird) models using rma function -#
rema.dl <- rma(yi=diff, sei=seDiff, data=ator80, method="DL", slab=trial)
confint(rema.dl, control=list(tau2.max=1000))
coef(rema.dl)

#- Fit RE (REML) models using rma function -#
rema.reml <- rma(yi=diff, sei=seDiff, data=ator80, method="REML", slab=trial)
confint(rema.reml, control=list(tau2.max=1000))
coef(rema.reml)

#-- Make forest plot with FE estimate added ---#
forest(x=ator80$diff, sei=ator80$seDiff, 
	slab=ator80$trial, refline=NULL, 
	at=seq(-80,-30,by=10),ylim=c(-2.5,nrow(ator80)+3),
	xlab="Mean Difference",
	main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-120.5, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)
addpoly(x=feEst, sei=seFeEst, mlab="Fixed Effects Estimate", row=-1)

#--------- Question 5 ---------#
forest(rema.reml, order=rev(order(ator80$diff)), 
	ylim=c(-2.5,nrow(ator80)+3), refline=NA, 
	mlab="Random Effects Estimate",
	main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-120.5, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)
addpoly(feEst, vi=1/sum(w), rows=-2, mlab="Fixed Effects Estimate",
	col='red')






