modelName <- "handsOn3" # root names of model file

if(.Platform$OS.type == "windows") Sys.setenv(HOME = "c:")
courseDir <- file.path(Sys.getenv("HOME"), "project/MI260")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- courseDir # change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
wineBin <- "/usr/local/MacPorts/bin" # directory containing wine and winepath programs.
                                        # only relevant on unix or Mac OS X
setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) # not required but assures repeatable results

load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## Get data file
adasData <- read.csv(file.path(exampleDir, "adasCogData.csv"), as.is = TRUE)

## rename and reformat MCMC results to facilitate later calculations and plots
sims.array = bugs.fit$sims.array
posterior = array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
  dimnames=list(NULL,dimnames(sims.array)[[3]]))

parametersToPlot <- c("alphaMean", "aucPlacebo", "kel", "keq", "beta",
                      "gamma", "eStar", "b", "eDelta", "et50", "sigma", "aResid",
                      "omegaInterceptStudy", "omegaInterceptUnit",  "omegaAlphaStudy",    
                      "omegaAlphaUnit", "omegaDrug", "dfInterceptUnit",
                      "dfAlphaUnit")

posterior <- posterior[,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
	paste("^",parametersToPlot,"\\[",sep="")),grep,x=dimnames(posterior)[[2]]))]

p <- apply(posterior, 2, median, na.rm = TRUE)


dose <- c(0, 5, 30, 60, 120)
drug <- c("placebo", "donepezil", "metupezil", "metupezil", "metupezil")
iDrug <- c(0, 1, 4, 4, 4)
n <- c(41, 33, 44, 39, 39)
time <- c(6, 12, 18, 24)
nTot <- sum(n)
nDoses <- length(dose)
nTimes <- length(time)


design <- data.frame(ID = rep(1:nTot, ea = nTimes),
                     WEEK = rep(time, nTot),
                     DRGN = rep(drug, nTimes * n),
                     DRUG = rep(iDrug, nTimes * n),
                     DTOT = rep(dose, nTimes * n))

simAdasCogCFB <- function(p, design){
  
  ## Simulate random effects. Results in posterior predictions for
  ## new observations in new patients with the same covariate values
  doseRef = c(donepezil = 5,            # reference doses
    galantamine = 24,
    rivastigmine = 6,
    metupezil = 120)

  ## study-level random effects:
  etaInterceptStudyPred <- 0
  etaDrugPred <- 0
  alphaStudyPred <- p["alphaMean"]

  ## residual variance:
  sigmaStudyPred <- p["sigma"] / 2

  nUnits <- length(unique(design$ID))
  ## patient-level random effects:
  etaInterceptPred <- rt(nUnits, p["dfInterceptUnit"]) * p["omegaInterceptUnit"] + 
    etaInterceptStudyPred
  alphaPred <- rt(nUnits, p["dfAlphaUnit"]) * p["omegaAlphaUnit"] + 
    alphaStudyPred

  time <- design$WEEK
  drug <- design$DRUG
  dose <- design$DTOT
  unit <- design$ID

  ## drug per se can't be used as a subscript because it includes
  ## zero. Therefore, we recode placebo from 0 to 1. Don't worry,
  ## drug effect gets zeroed out for placebo anyway (see eDrugPred)
  iDrug <- ifelse(drug == 0, 1, drug)
   
  ## these parameters are vectors with one element per drug:
  gamma <- c(p[grep("^gamma\\[", names(p))], p["gamma[1]"])
  eDelta <- c(p[grep("^eDelta\\[", names(p))], 1.5 * p["eDelta[1]"])
  et50 <- c(p[grep("^et50\\[", names(p))], p["et50[1]"])

  ##
  ## Guts of the model:
  ##
    
  ePlacebo <- p["beta"] * (exp(-p["kel"]*time) - exp(-p["keq"]*time))
    
  eDrugPred <- ifelse(drug == 0,
                      yes = 0,          # zero drug effect for placebo
                      no = ((dose / doseRef[iDrug]) ^ gamma[iDrug] *
                            eDelta[iDrug] * exp(etaDrugPred) *
                            time / (et50[iDrug] + time)
                            )
                      )
    
  adasCFBMeanPred <- (alphaPred[unit] * time + etaInterceptPred[unit] + 
                      ePlacebo + eDrugPred) 
    
  ##
  ## End guts of model
  ##

  rnorm(length(adasCFBMeanPred), adasCFBMeanPred, sigmaStudyPred)
}

design$ADAS <- simAdasCogCFB(p, design)

xyplot(ADAS ~ WEEK | as.factor(DTOT), design, groups = ID,
       type = "l")

write.csv(design, file.path(courseDir, "handsOn4", "metupezil.csv"), row.names = FALSE)

