modelName <- "adasCog" # root names of model file

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(.Platform$OS.type == "windows") 
   bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
wineBin <- "/usr/local/MacPorts/bin" # directory containing wine and winepath programs.
                                        # only relevant on unix or Mac OS X
setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
library(gdata)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

set.seed(10271998) # not required but assures repeatable results

extend.limits <- lattice:::extend.limits

## Get data file
adasData <- read.csv(file.path(exampleDir, "adasCogData.csv"), as.is = TRUE)

## Create sequential indices for study and treatment arm
adasData$studyID <- as.numeric(as.factor(adasData$study))
adasData$unitID <- as.numeric(as.factor(adasData$NMID))

## These vectors are used by the unit loop in bugs:
studyUnit <- adasData$studyID[ !duplicated(adasData$unitID) ]
nPatUnit <- adasData$NTRT[ !duplicated(adasData$unitID) ]

studyAge <- 2009 - adasData$PY[ !duplicated(adasData$studyID) ] # study age in years

## Create integer treatment identifier
drugs <- c("donepezil", "galantamine", "rivastigmine")
treatments <- c("placebo", drugs)
numericTreatments <- as.numeric(swap(tolower(adasData$DRGN), 
                                     oldval = treatments,
                                     newval = seq(treatments) - 1 )
                                )

bugsdata <- list(
                 nStudies = length(unique(adasData$studyID)),
                 nUnits = length(unique(adasData$unitID)),
                 nDrugs = length(treatments) - 1,
                 nObs = nrow(adasData),                 
                 nPat = adasData$NTRT,
                 nPatUnit = nPatUnit,
                 study = adasData$studyID,
                 studyUnit = studyUnit,  
                 unit = adasData$unitID, # in this case, each unit is a study arm 
                 time = adasData$WEEK,
                 drug = numericTreatments, # recall: indexed from 0 to n(treatments) - 1
                 dose = adasData$DTOT, 
                 adasCFB = adasData$ADAS,
                 varAdasCFB = adasData$sd^2,
                 tStar = 12,            # reference time = 12 weeks
                 doseRef = c(donepezil = 5, # reference doses
                   galantamine = 24,
                   rivastigmine = 6)
                 )

bugsinit <- function() {
    
  qnorm.trunc <- function(p,mean=0,sd=1,lower=-Inf,upper=Inf)
    qnorm(p*pnorm(upper,mean,sd)+(1-p)*pnorm(lower,mean,sd),mean,sd)
    
  rnorm.trunc <- function(n,mean=0,sd=1,lower=-Inf,upper=Inf)
    qnorm.trunc(runif(n),mean,sd,lower,upper)

  nStudies <- bugsdata$nStudies
  nUnits <- bugsdata$nUnits
    
  x1 <- list(
             ## overall population mean for alpha: 
             alphaMean = rnorm(1, 0.12, 0.05),

             ## placebo effect parameters:
             aucPlacebo = exp(rnorm(1, log(2), 0.4)), 
             kel = exp(rnorm(1, log(0.0427), 0.2)),
             keqMinusKel = exp(rnorm(1, log(0.0724 - 0.0427), 0.2)),

             ## drug effect parameters:
             gamma = rnorm.trunc(3, c(donepezil = 0.28, galantamine = 0.28,
               rivastigmine = 0.82), 0.5, lower = 0),
             logEStar = rnorm(3, log(2), 0.5),
             b = rnorm.trunc(3, 12 / c(2.6, 17, 5), 4, lower = 0),

             ## residual variance:
             sigma = rnorm.trunc(1, 3.1, 1, lower = 0),
             aResid = exp(rnorm(1, log(0.7), 0.2)),

             ## variance components (and related df):
             omegaInterceptStudy = rnorm.trunc(1, 0.65, 0.1, lower = 0),
             omegaInterceptUnit = rnorm.trunc(1, 3, 0.1, lower = 0),
             omegaAlphaStudy = rnorm.trunc(1, 0.04, 0.01, lower = 0),
             omegaAlphaUnit = rnorm.trunc(1, 0.05, 0.01, lower = 0),
             omegaDrug = rnorm.trunc(1, 0.04, 0.01, lower = 0),
               
             dfInterceptUnit = 5,
             dfAlphaUnit = 5
             )

  x2 <- list( 
             ## random effects:
             etaInterceptStudy = rep(0, nStudies),
             alphaStudy = rep(x1$alphaMean, nStudies),
             etaDrug = rep(0, nStudies),
             tauStudy = rep(1 / (x1$sigma^2), nStudies),
             etaIntercept = rep(0, nUnits),
             alpha = rep(x1$alphaMean, nUnits)
             )

  inits <- c(x1, x2)
  return(inits)
    
}

## Specify the variables for which you want history and density plots
parametersToPlot <- c("alphaMean", "aucPlacebo", "kel", "keq", "beta",
                      "gamma", "eStar", "b", "eDelta", "et50", "sigma", "aResid",
                      "omegaInterceptStudy", "omegaInterceptUnit",  "omegaAlphaStudy",    
                      "omegaAlphaUnit", "omegaDrug", "dfInterceptUnit",
                      "dfAlphaUnit")

## Additional variables to monitor
otherRVs <- c("adasCFBCond", "varAdasCFBCond", "adasCFBPred", "varAdasCFBPred",
              "etaInterceptStudy", "alphaStudy", "etaDrug", "etaIntercept", "alpha", "sigmaStudy")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
# run WinBUGS

n.chains <- 3
n.iter <- 10000
n.burnin <- 5000
n.thin <- 5
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = TRUE, debug = TRUE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory
save.model(bugs.fit, modelName, n.chains = n.chains)
##load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots
sims.array = bugs.fit$sims.array
posterior = array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
  dimnames=list(NULL,dimnames(sims.array)[[3]]))
                                 
################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 = sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
	paste("^",parametersToPlot,"\\[",sep="")),grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable = parameter.plot.table(x1)
write.csv(ptable, paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
# posterior predictive distributions

# prediction of future observations in the same studies and patients, i.e., posterior predictions
# conditioned on observed data from the same study

pred <- posterior[,grep("adasCFBCond\\[",dimnames(posterior)[[2]])]
predSD <- sqrt(posterior[,grep("varAdasCFBCond\\[",dimnames(posterior)[[2]])])

## Calculate posterior median, 5th and 95th percentiles for each treatment arm

x1 <- adasData
x2 <- rbind(x1,x1,x1)
x2$ADAS <- as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$sd <- as.vector(t(apply(predSD,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type <- rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1$type <- rep("observed",nrow(x1))
plotVals <- rbind(x1,x2)

for(tMax in c(Inf, 12)){
  
  useVals <- plotVals[plotVals$WEEK <= tMax, ]
  
  for(drug in c("donepezil", "galantamine", "rivastigmine")) {

    studies <- unique(useVals$study)
    drug.in.study <- sapply(studies,
                            function(study, drug, xdata) {
                              drug %in% unique(xdata$DRGN[xdata$study == study])
                            },
                            drug = drug, xdata = useVals)
    studies <- studies[drug.in.study]
    xdata <- subset(useVals, (study %in% studies) & (DRGN %in% c(drug, "placebo")))
    xdata$treatment <- reorder.factor(as.factor(paste(xdata$DRGN, xdata$DTOT)))
    treatments <- levels(xdata$treatment)
    ntreat <- length(treatments)

    ## The factor xdelta is used in the panel function below to extend
    ## the median and error bar lines when necessary to make them more
    ## visible.
    xdelta <- diff(extend.limits(range(xdata$WEEK))) / 7

    print(xyplot(ADAS ~ WEEK | as.factor(study),
                 data = xdata, 
                 groups = reorder.factor(as.factor(paste(DRGN, DTOT, type))), 
                 panel = panel.superpose,
                 xdelta = xdelta,
                 panel.groups = function(x, y, group.number,
                   xdelta, subscripts, ...) {
                   trt <- xdata$NMID[subscripts]
                   trts <- unique(trt)
                   for(itrt in trts) {
                     xx <- x[trt == itrt]
                     yy <- y[trt == itrt]
                     if(length(xx) == 1 & (group.number %% 4) != 0) {
                       ## This clause expands the xrange for the
                       ## median and error bars in the case when there
                       ## is only one x value. (The modulo 4 condition
                       ## makes sure that you don't expand the x value
                       ## on the observed value itself).
                       x2 <- xx + c(-1, 1) * xdelta
                       y2 <- rep(yy, 2)
                     } else{
                       x2 <- xx
                       y2 <- yy
                     }
                     panel.xyplot(x2, y2, ...)
                   }
                 },
                 main = paste("individual predictions:", drug), 
                 key = list(space = "right", type = "b", pch = 19, lwd = 2,
                   text = list(label = treatments, cex = 1.2),
                   lines = list(col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                                  "purple", "grey", "brown", "light blue")[1:ntreat])),
                 type = c("l", "l", "l", "p"),
                 pch = c(NA, NA, NA, 19),
                 lwd = 2,
                 lty = c(2, 2, 1, 0),
                 col = rep(c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                   "purple", "grey", "brown", "light blue"), ea = 4),
                 cex = 0.8,
                 scales = list(cex=1.2),
                 xlab = list(label="time (w)", cex=1.2),
                 ylab = list(label="mean ADAS-cog change from baseline", cex=1.2),
                 par.strip.text = list(cex=1.2)
                 )
          )

    
    print(xyplot(sd ~ WEEK | as.factor(study), xdata, 
                 groups = reorder.factor(as.factor(paste(DRGN,DTOT,type))),
                 panel = panel.superpose,
                 xdelta = xdelta,
                 panel.groups = function(x, y, group.number, xdelta, subscripts, ...) {
                   trt <- xdata$NMID[subscripts]
                   trts <- unique(trt)
                   for(itrt in trts){
                     xx <- x[trt == itrt]
                     yy <- y[trt == itrt]
                     if(length(xx) == 1 & (group.number %% 4) != 0){
                       ## See comments for preceeding plot to
                       ## understand what this is doing.
                       x2 <- xx + c(-1, 1) * xdelta
                       y2 <- rep(yy, 2)
                     } else {
                       x2 <- xx
                       y2 <- yy
                     }
                     panel.xyplot(x2, y2, ...)
                   }
                 },
                 main = paste("individual predictions:", drug), 
                 key = list(space = "right", type = "b", pch = 19, lwd = 2,
                   text = list(label = treatments, cex = 1.2),
                   lines = list(col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                                  "purple", "grey", "brown", "light blue")[1:ntreat])),
                 type = c("l","l","l","p"), pch = c(NA,NA,NA,19), lwd = 2, lty = c(2,2,1,0),
                 col = rep(c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                   "purple", "grey", "brown", "light blue"), ea = 4),
                 cex=0.8,scales=list(cex=1.2),
                 xlab=list(label="time (w)",cex=1.2),
                 ylab=list(label="SD ADAS-cog change from baseline",cex=1.2),
                 par.strip.text=list(cex=1.2)
                 )
          )

  } # end loop over drugs
  
} # end loop over time cut-offs

################################################################################################
# posterior predictive distributions

# prediction of future observations in new studies and patients using the same design

pred <- posterior[,grep("adasCFBPred\\[",dimnames(posterior)[[2]])]
predSD <- sqrt(posterior[,grep("varAdasCFBPred\\[",dimnames(posterior)[[2]])])

## Calculate posterior median, 5th and 95th percentiles for each treatment arm

x1 <- adasData
x2 <- rbind(x1,x1,x1)
x2$ADAS <- as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$sd <- as.vector(t(apply(predSD,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type <- rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1$type <- rep("observed",nrow(x1))
plotVals <- rbind(x1,x2)


for(tMax in c(Inf, 12)){
  
  useVals <- plotVals[plotVals$WEEK <= tMax, ]
  
  for(drug in c("donepezil", "galantamine", "rivastigmine")) {

    studies <- unique(useVals$study)
    drug.in.study <- sapply(studies,
                            function(study, drug, xdata) {
                              drug %in% unique(xdata$DRGN[xdata$study == study])
                            },
                            drug = drug, xdata = useVals)
    studies <- studies[drug.in.study]
    xdata <- subset(useVals, (study %in% studies) & (DRGN %in% c(drug, "placebo")))
    xdata$treatment <- reorder.factor(as.factor(paste(xdata$DRGN, xdata$DTOT)))
    treatments <- levels(xdata$treatment)
    ntreat <- length(treatments)

    ## The factor xdelta is used in the panel function below to extend
    ## the median and error bar lines when necessary to make them more
    ## visible.
    xdelta <- diff(extend.limits(range(xdata$WEEK))) / 7

    print(xyplot(ADAS ~ WEEK | as.factor(study),
                 data = xdata, 
                 groups = reorder.factor(as.factor(paste(DRGN, DTOT, type))), 
                 panel = panel.superpose,
                 xdelta = xdelta,
                 panel.groups = function(x, y, group.number,
                   xdelta, subscripts, ...) {
                   trt <- xdata$NMID[subscripts]
                   trts <- unique(trt)
                   for(itrt in trts) {
                     xx <- x[trt == itrt]
                     yy <- y[trt == itrt]
                     if(length(xx) == 1 & (group.number %% 4) != 0) {
                       ## This clause expands the xrange for the
                       ## median and error bars in the case when there
                       ## is only one x value. (The modulo 4 condition
                       ## makes sure that you don't expand the x value
                       ## on the observed value itself).
                       x2 <- xx + c(-1, 1) * xdelta
                       y2 <- rep(yy, 2)
                     } else{
                       x2 <- xx
                       y2 <- yy
                     }
                     panel.xyplot(x2, y2, ...)
                   }
                 },
                 main = paste("population predictions:", drug), 
                 key = list(space = "right", type = "b", pch = 19, lwd = 2,
                   text = list(label = treatments, cex = 1.2),
                   lines = list(col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                                  "purple", "grey", "brown", "light blue")[1:ntreat])),
                 type = c("l", "l", "l", "p"),
                 pch = c(NA, NA, NA, 19),
                 lwd = 2,
                 lty = c(2, 2, 1, 0),
                 col = rep(c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                   "purple", "grey", "brown", "light blue"), ea = 4),
                 cex = 0.8,
                 scales = list(cex=1.2),
                 xlab = list(label="time (w)", cex=1.2),
                 ylab = list(label="mean ADAS-cog change from baseline", cex=1.2),
                 par.strip.text = list(cex=1.2)
                 )
          )
    
    print(xyplot(sd ~ WEEK | as.factor(study), xdata, 
                 groups = reorder.factor(as.factor(paste(DRGN,DTOT,type))),
                 panel = panel.superpose,
                 xdelta = xdelta,
                 panel.groups = function(x, y, group.number, xdelta, subscripts, ...) {
                   trt <- xdata$NMID[subscripts]
                   trts <- unique(trt)
                   for(itrt in trts){
                     xx <- x[trt == itrt]
                     yy <- y[trt == itrt]
                     if(length(xx) == 1 & (group.number %% 4) != 0){
                       ## See comments for preceeding plot to
                       ## understand what this is doing.
                       x2 <- xx + c(-1, 1) * xdelta
                       y2 <- rep(yy, 2)
                     } else {
                       x2 <- xx
                       y2 <- yy
                     }
                     panel.xyplot(x2, y2, ...)
                   }
                 },
                 main = paste("population predictions:", drug), 
                 key = list(space = "right", type = "b", pch = 19, lwd = 2,
                   text = list(label = treatments, cex = 1.2),
                   lines = list(col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                                  "purple", "grey", "brown", "light blue")[1:ntreat])),
                 type = c("l","l","l","p"), pch = c(NA,NA,NA,19), lwd = 2, lty = c(2,2,1,0),
                 col = rep(c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                   "purple", "grey", "brown", "light blue"), ea = 4),
                 cex=0.8,scales=list(cex=1.2),
                 xlab=list(label="time (w)",cex=1.2),
                 ylab=list(label="SD ADAS-cog change from baseline",cex=1.2),
                 par.strip.text=list(cex=1.2)
                 )
          )

  } # end loop over drugs
  
} # end loop over time cut-offs

######################################################################
### Various internal and external checks on the posterior
######################################################################

qqhist2 <- function(x, label = NULL, main=NULL, distribution = qnorm)
{
  ## plot histogram and quantile-normal quantile plot for vector x
  
  plot1 <- histogram( ~ x,
                     ylab = list(cex=1.2),
                     xlab = list(label=label,cex=1.2),
                     scales = list(cex=1.2))
  plot2 <- qqmath( ~ x,
                  prepanel = prepanel.qqmathline,
                  distribution = distribution,
                  panel = function(x, distribution){
                    panel.qqmath(x,cex=1.2,pch=16, distribution = distribution)
                    panel.qqmathline(x,distribution=distribution,col=3,lwd=3)
                  },
                  ylab = list(label=label,cex=1.2),
                  xlab = list(cex=1.2),
                  scales = list(cex=1.2),ylim=range(x))
  list(plot1 = plot1, plot2 = plot2)

}

######################################################################

## note etaDrug models between-study variation in drug effects (it
## does not model between-drug variation)
studyEffects <- c("etaInterceptStudy", "alphaStudy", "etaDrug") 

unitEffects <- c("etaIntercept", "alpha")

unitDF <- c("dfInterceptUnit", "dfAlphaUnit")

######################################################################
### Investigate study effects

effects <- studyEffects

studyInfo <- data.frame(study = bugsdata$study,
                        studyAge = 2008 - adasData$PY,
                        statType = adasData$STATANAL
                        )


studyInfo <- studyInfo[!duplicated(studyInfo$study), ]
studyInfo$dose <- as.numeric(as.character(studyInfo$dose))
studyInfo$studyAge <- as.numeric(as.character(studyInfo$studyAge))

effectVars <- grep(paste(effects[1],"\\[",sep=""),
                   dimnames(posterior)[[2]], value = TRUE)

## study x parameter-type matrix of posterior medians:
medianEffects <- sapply(effects,
                        function(effect, posterior){
                          effectCols <- grep(paste(effect,"\\[",sep=""),
                                             dimnames(posterior)[[2]])
                          apply(posterior[, effectCols], 2, median)
                        },
                        posterior = posterior
                        )

## study x parameter-type matrix of posterior means:
meanEffects <- sapply(effects,
                      function(effect, posterior){
                        effectCols <- grep(paste(effect,"\\[",sep=""),
                                           dimnames(posterior)[[2]])
                        apply(posterior[, effectCols], 2, mean)
                      },
                      posterior = posterior
                      )

for(i in 1:ncol(medianEffects)) {

  ## medianDF = median(posterior[,dfs[i]])
  plots <- qqhist2(medianEffects[, i],
                   label = paste("median", effects[i]),
                   distribution = qnorm)
  ## function(x) qt(x, df=medianDF))
  print(plots[[1]], position = c(0, 0, 0.525, 1), more = TRUE)
  print(plots[[2]], position = c(0.475, 0, 1, 1), more = FALSE)
  ## Note medianEffects is aligned with a column subset of posterior,
  ## and that column subset is aligned with the unique values of the
  ## 'study' element of bugsdata, which is the same as the unique
  ## values of studyInfo$study
  print(medianEffects[studyInfo$study == 56, i]) # CP-457,920
  print(medianEffects[studyInfo$study == 57, i]) # LEADe
  print(summary(medianEffects[!(studyInfo$study %in% c(56,57)), i] )) # literature summary data
  
}

sigmaStudyCols <- grep("sigmaStudy\\[", dimnames(posterior)[[2]]) 

tau <- apply(1 / posterior[, sigmaStudyCols]^2, 2, median)

plots <- qqhist2(tau, label = paste("median","tauStudy"),
                 distribution = function(x) {
                   qgamma(x, shape = median(posterior[, "aResid"]),
                          rate = median(posterior[, "aResid"] * posterior[, "sigma"]^2))
                 }
                 )
               
print(plots[[1]], position=c(0, 0, 0.525, 1), more = TRUE)
print(plots[[2]], position=c(0.475, 0, 1, 1), more = FALSE)

pairs(medianEffects)

xyplot(medianEffects[1:nrow(studyInfo), "alphaStudy"] ~ studyAge,
       data = studyInfo,
       panel = function(x,y,...){
         panel.xyplot(x,y,...)
         panel.abline(h = 0, lwd = 2, lty = 2)
         panel.lmline(x,y,...)
         print(summary(lm(y~x)))
       },
       xlab=list(label="study age (y)",cex=1.5),
       ylab=list(label="study-specific slope (units/week)", cex=1.5),
       scales=list(cex=1.5), lwd=2, cex=1.2
       )

xyplot(medianEffects[1:nrow(studyInfo), "alphaStudy"] ~ as.factor(statType),
       data = studyInfo,
       panel = function(x,y,...){
         panel.xyplot(x,y,...)
         panel.abline(h = 0, lwd = 2, lty = 2)
       },
       xlab=list(label="statistic type",cex=1.5),
       ylab=list(label="study-specific slope (units/week)", cex=1.5),
       scales=list(cex=1.5, x = list(rot = 45)),
       cex=1.2
       )

### End investigate study effects
######################################################################


######################################################################
### Invesigate unit effects

effects <- unitEffects
dfs <- unitDF

unitInfo <- data.frame(study=bugsdata$study,
                       unit=bugsdata$unit,
                       drug=swap(bugsdata$drug, 0:3, 
                         c("placebo", "donepezil", "galantamine",
                           "rivastigmine")),
                       dose=bugsdata$dose
                      )

unitInfo <- unitInfo[!duplicated(unitInfo$unit), ]
unitInfo$dose <- as.numeric(as.character(unitInfo$dose))
unitInfo$adas0 <- as.numeric(as.character(unitInfo$adas0))

medianEffects <- sapply(effects,
                        function(effect, posterior){
                          effectCols <- grep(paste(effect,"\\[", sep=""),
                                             dimnames(posterior)[[2]])
                          apply(posterior[, effectCols],
                                2, median)
                        },
                        posterior = posterior
                        )

for(i in 1:ncol(medianEffects)) { 
  medianDF <- median(posterior[ , dfs[i] ])
  plots <- qqhist2(medianEffects[, i],
                   label = paste("median", effects[i]),
                   distribution = function(x) qt(x, df = medianDF)
                   )
  print(plots[[1]], position = c(0,0,0.525,1), more = TRUE)
  print(plots[[2]], position=c(0.475,0,1,1), more = FALSE)
  
}

for(i in 1:ncol(medianEffects)) {
  medianDF <- median(posterior[, dfs[i] ])
  print(qqmath( ~ medianEffects[1:nrow(unitInfo),i] | drug,
               data = unitInfo,
               prepanel = prepanel.qqmathline, 
               distribution = function(x) qt(x, df = medianDF),
               panel = function(x, distribution){
                 panel.qqmath(x, cex = 1.2, pch = 16, distribution = distribution)
                 panel.qqmathline(x, distribution = distribution, col=3, lwd=3)
               },
               ylab = list(label = effects[i], cex=1.2),
               xlab = list(cex = 1.2),
               scales = list(cex = 1.2),
               ylim = range(medianEffects[1:nrow(unitInfo), i])
               )
        )
}

print(summary(medianEffects[, i])) 


pairs(medianEffects)

for(i in 1:ncol(medianEffects)){

  print(xyplot(medianEffects[1:nrow(unitInfo), i] ~ dose | as.factor(drug),
               data = unitInfo,
               panel = function(x,y,...){
                 panel.xyplot(x,y,...)
                 panel.abline(h = 0, lwd = 2, lty = 2)
                 if(length(unique(x)) > 1) panel.lmline(x,y,...)
               },
               xlab = list(label="dose", cex=1.2),
               ylab = list(label=effects[i], cex=1.2),
               scales=list(x=list(relation = "free"))
               )
        )
	
}

### End investigate unit effects
######################################################################

dev.off()

