modelName <- "statinGemcabeneTruncEmax2" # root names of model file
## Just like statinGemcabeneTruncEmax except using observed SE values

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI260W")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup
bugsDir <- "c:/Program Files/WinBUGS14"
if(.Platform$OS.type == "windows") 
   bugsDir <- file.path(Sys.getenv("HOME"), "Program Files/WinBUGS14")
wineBin <- "/opt/local/bin" # directory containing wine and winepath programs.
                                        # only relevant on unix or Mac OS X
setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))
source(file.path(courseDir, "mbmaPpcPlot.R"))

## Get data files
ldlData <- read.csv(file.path(exampleDir, "ldlData.csv"), as.is = TRUE)

## SE values are missing for some observations. They are required for this analysis method, so
## we either need to exclude means without a corresponding SE value or impute a SE. Fir this
## example I will just exclude them.
ldlData <- ldlData[!is.na(ldlData$seLdlPcfb),]

## Create integer code for statin and nonstatin drugs
## statins: atorvastatin = 1, rosuvastatin = 2, simvastatin = 3, lovastatin = 4, pravastatin = 5
## non-statins: ezetimibe = 1, gemcabene = 2
ldlData$iStatin <- as.numeric(swap(ldlData$drug1,
                        c("None", "Atorvastatin", "Rosuvastatin", "Simvastatin",
                          "Lovastatin", "Pravastatin"),
                        c(1, 1:5)))
ldlData$iNonStatin <- as.numeric(swap(ldlData$drug2, c("None", "Ezetimibe", "Gemcabene"), c(1, 1, 2)))

## Create sequential index for trial
ldlData$iTrial <- as.numeric(as.factor(ldlData$trial))

## Format data for WinBUGS
bugsdata <- list(
                 nArms = nrow(ldlData),
                 nTrials = max(ldlData$iTrial),
                 nStatins = max(ldlData$iStatin),
                 nNonStatins = max(ldlData$iNonStatin),
                 trial = ldlData$iTrial,
                 statin = ldlData$iStatin,
                 nonStatin = ldlData$iNonStatin,
                 doseStatin = ldlData$dose1,
                 doseNonStatin = ldlData$dose2,
                 pcldl = ldlData$ldlPcfb,
                 sepcldl = ldlData$seLdlPcfb,
                 dStarStatin = 80,
                 dStarNonStatin = c(40, 900)
                 )

## Create initial estimates
bugsinit <- function() 
  list(
       e0Mean = rnorm(1, 0, 2),
       eStarStatin = exp(rnorm.trunc(1, log(60), 0.4, upper = log(100))),
       bStatin = c(exp(rnorm(1, log(80 / 10), 0.4)), NA, NA, NA, NA),
       logPiStatin = c(NA, rnorm(4, log(c(2, 0.5, 0.125, 0.125)), 0.4)),
       nStatin = exp(rnorm.trunc(1, log(0.5), 0.4, log(0.1), log(10))),
       eStarNonStatin = exp(rnorm.trunc(2, log(c(20, 30)), 0.4, upper = log(100))),
       bNonStatin = exp(rnorm(2, log(c(40/0.5, 900/400)), 0.4)),
       nNonStatin = c(NA, exp(rnorm.trunc(1, log(2), 0.4, log(0.1), log(10)))),
       logGamma = rnorm(2, log(c(1, 2)), 0.4),
       e0Prec = 1 / exp(2 * rnorm(1, log(3), 0.4))
       )


## Specify the variables for which you want history and density plots
parametersToPlot <- c(
                      "e0Mean", "emaxStatin", "ed50Statin", "nStatin",
                      "emaxNonStatin", "ed50NonStatin", "nNonStatin",
                      "gamma", "omegaE0",
                      "eStarStatin", "bStatin", "piStatin",
                      "eStarNonStatin", "bNonStatin"
                      )

## Additional variables to monitor
otherRVs <- c("pcldlCond", "pcldlPred", "pcldlMeanPred", "e0")

parameters <- c(parametersToPlot, otherRVs)
parametersToPlot <- c("deviance", parametersToPlot)

################################################################################################
# run WinBUGS

n.chains <- 3
n.iter <- 10000
n.burnin <- 4000
n.thin <- 5
bugs.fit <- bugs(data = bugsdata, inits = bugsinit, parameters.to.save = parameters,
                 model.file = file.path(exampleDir, paste(modelName, ".txt", sep="")),
                 n.chains = n.chains, n.iter = n.iter, n.burnin = n.burnin,
                 n.thin = n.thin, clearWD = FALSE, debug = TRUE,
                 bugs.directory = bugsDir, working.directory = getwd(),
                 useWINE = (.Platform$OS.type == "unix"),
                 WINE = file.path(wineBin,"wine"),
                 newWINE = (.Platform$OS.type == "unix"),
                 WINEPATH = file.path(wineBin,"winepath"))

## save scripts, data and results to a directory
save.model(bugs.fit, modelName, n.chains = n.chains)
##load(paste(modelName,"/",modelName,".fit.Rsave",sep=""))

## rename and reformat MCMC results to facilitate later calculations and plots
sims.array = bugs.fit$sims.array
posterior = array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
  dimnames=list(NULL,dimnames(sims.array)[[3]]))
                                 
################################################################################################
## posterior distributions of parameters

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"/",modelName,"Plots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

## subset of sims.array containing selected variables
x1 = sims.array[,,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
	paste("^",parametersToPlot,"\\[",sep="")),grep,x=dimnames(sims.array)[[3]]))]

## create history, density and Gelman-Rubin-Brooks plots, and a table of summary stats
ptable = parameter.plot.table(x1)
write.csv(ptable, paste(modelName,"/",modelName,".summary.csv",sep=""))

################################################################################################
# posterior predictive distributions

# prediction of future observations in the same studies, i.e., posterior predictions
# conditioned on observed data from the same study

pred = posterior[,grep("pcldlCond\\[",dimnames(posterior)[[2]])]

## Calculate posterior median, 5th and 95th percentiles for each treatment arm

x1 = ldlData
x2 = rbind(x1,x1,x1)
x2$ldlPcfb = as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1$type = rep("observed",nrow(x1))
x1 = rbind(x1,x2)

## Rename 0 dose treatments as "Placebo"
x1$drug1 <- ifelse(x1$dose1 == 0, "Placebo", x1$drug1)
x1$drug2 <- ifelse(x1$dose2 == 0, "Placebo", x1$drug2)

## Plots for statin monotherapy
drugs <- sort(unique(x1$drug1))
drugs <- drugs[drugs != "Placebo"]
xplot <- x1[x1$dose2 == 0,]
for(drug in drugs){
  yplot <- xplot[xplot$drug1 %in% c(drug, "Placebo"),]
  doses <- sort(unique(yplot$dose1))
  nDoses <- length(doses)
  yplot$treatment <- ordered(paste(yplot$drug1, yplot$dose1, "mg"),
                             levels = paste(c("Placebo", rep(drug, nDoses)),
                               doses, "mg"))
  ppcPlot <- mbmaPpcPlot(data = yplot, x = "ldlPcfb",
                     xlab = list(label = "LDL % change from baseline", cex = 1.2))
  print(update(ppcPlot, main = list(label = "Individual Predictions", cex = 1.2)))
}     

## Plots for non-statin monotherapy

drugs <- sort(unique(x1$drug2))
drugs <- drugs[drugs != "Placebo"]
xplot <- x1[x1$dose1 == 0,]
for(drug in drugs){
  yplot <- xplot[xplot$drug2 %in% c(drug, "Placebo"),]
  doses <- sort(unique(yplot$dose2))
  nDoses <- length(doses)
  yplot$treatment <- ordered(paste(yplot$drug2, yplot$dose2, "mg"),
                             levels = paste(c("Placebo", rep(drug, nDoses)),
                               doses, "mg"))
  ppcPlot <- mbmaPpcPlot(data = yplot, x = "ldlPcfb",
                     xlab = list(label = "LDL % change from baseline", cex = 1.2))
  print(update(ppcPlot, main = list(label = "Individual Predictions", cex = 1.2)))
}     

## Plots for statin/non-statin combinations

## Ezetimibe combinations
combTrials <- unique(x1$trial[x1$dose1 > 0 & x1$dose2 > 0 & x1$drug2 == "Ezetimibe"])
xplot <- x1[x1$trial %in% combTrials,]
drugs <- sort(unique(xplot$drug1))
drugs <- drugs[drugs != "Placebo"]

for(drug in drugs){
  yplot <- xplot[xplot$drug1 %in% c(drug, "Placebo"),]
  yplot$treatment <- paste(yplot$drug1, yplot$dose1, "mg /",
                          yplot$drug2, yplot$dose2, "mg")
  ppcPlot <- mbmaPpcPlot(data = yplot, x = "ldlPcfb",
                     xlab = list(label = "LDL % change from baseline", cex = 1.2))
  print(update(ppcPlot, main = list(label = "Individual Predictions", cex = 1.2)))
}     

## Gemcabene combinations
combTrials <- unique(x1$trial[x1$dose1 > 0 & x1$dose2 > 0 & x1$drug2 == "Gemcabene"])
xplot <- x1[x1$trial %in% combTrials,]
drugs <- sort(unique(xplot$drug1))
drugs <- drugs[drugs != "Placebo"]

for(drug in drugs){
  yplot <- xplot[xplot$drug1 %in% c(drug, "Placebo"),]
  yplot$treatment <- paste(yplot$drug1, yplot$dose1, "mg /",
                          yplot$drug2, yplot$dose2, "mg")
  ppcPlot <- mbmaPpcPlot(data = yplot, x = "ldlPcfb",
                     xlab = list(label = "LDL % change from baseline", cex = 1.2))
  print(update(ppcPlot, main = list(label = "Individual Predictions", cex = 1.2)))
}     

################################################################################################
# posterior predictive distributions

# prediction of future observations in new studies of the same size

pred = posterior[,grep("pcldlPred\\[",dimnames(posterior)[[2]])]

## Calculate posterior median, 5th and 95th percentiles for each treatment arm

x1 = ldlData
x2 = rbind(x1,x1,x1)
x2$ldlPcfb = as.vector(t(apply(pred,2,quantile,probs=c(0.05,0.5,0.95))))
x2$type = rep(c("5%ile","median","95%ile"),ea=nrow(x1))
x1$type = rep("observed",nrow(x1))
x1 = rbind(x1,x2)

## Rename 0 dose treatments as "Placebo"
x1$drug1 <- ifelse(x1$dose1 == 0, "Placebo", x1$drug1)
x1$drug2 <- ifelse(x1$dose2 == 0, "Placebo", x1$drug2)

## Plots for statin monotherapy
drugs <- sort(unique(x1$drug1))
drugs <- drugs[drugs != "Placebo"]
xplot <- x1[x1$dose2 == 0,]
for(drug in drugs){
  yplot <- xplot[xplot$drug1 %in% c(drug, "Placebo"),]
  doses <- sort(unique(yplot$dose1))
  nDoses <- length(doses)
  yplot$treatment <- ordered(paste(yplot$drug1, yplot$dose1, "mg"),
                             levels = paste(c("Placebo", rep(drug, nDoses)),
                               doses, "mg"))
  ppcPlot <- mbmaPpcPlot(data = yplot, x = "ldlPcfb",
                     xlab = list(label = "LDL % change from baseline", cex = 1.2))
  print(update(ppcPlot, main = list(label = "Population Predictions", cex = 1.2)))
}     

## Plots for non-statin monotherapy

drugs <- sort(unique(x1$drug2))
drugs <- drugs[drugs != "Placebo"]
xplot <- x1[x1$dose1 == 0,]
for(drug in drugs){
  yplot <- xplot[xplot$drug2 %in% c(drug, "Placebo"),]
  doses <- sort(unique(yplot$dose2))
  nDoses <- length(doses)
  yplot$treatment <- ordered(paste(yplot$drug2, yplot$dose2, "mg"),
                             levels = paste(c("Placebo", rep(drug, nDoses)),
                               doses, "mg"))
  ppcPlot <- mbmaPpcPlot(data = yplot, x = "ldlPcfb",
                     xlab = list(label = "LDL % change from baseline", cex = 1.2))
  print(update(ppcPlot, main = list(label = "Population Predictions", cex = 1.2)))
}     

## Plots for statin/non-statin combinations

## Ezetimibe combinations
combTrials <- unique(x1$trial[x1$dose1 > 0 & x1$dose2 > 0 & x1$drug2 == "Ezetimibe"])
xplot <- x1[x1$trial %in% combTrials,]
drugs <- sort(unique(xplot$drug1))
drugs <- drugs[drugs != "Placebo"]

for(drug in drugs){
  yplot <- xplot[xplot$drug1 %in% c(drug, "Placebo"),]
  yplot$treatment <- paste(yplot$drug1, yplot$dose1, "mg /",
                          yplot$drug2, yplot$dose2, "mg")
  ppcPlot <- mbmaPpcPlot(data = yplot, x = "ldlPcfb",
                     xlab = list(label = "LDL % change from baseline", cex = 1.2))
  print(update(ppcPlot, main = list(label = "Population Predictions", cex = 1.2)))
}     

## Gemcabene combinations
combTrials <- unique(x1$trial[x1$dose1 > 0 & x1$dose2 > 0 & x1$drug2 == "Gemcabene"])
xplot <- x1[x1$trial %in% combTrials,]
drugs <- sort(unique(xplot$drug1))
drugs <- drugs[drugs != "Placebo"]

for(drug in drugs){
  yplot <- xplot[xplot$drug1 %in% c(drug, "Placebo"),]
  yplot$treatment <- paste(yplot$drug1, yplot$dose1, "mg /",
                          yplot$drug2, yplot$dose2, "mg")
  ppcPlot <- mbmaPpcPlot(data = yplot, x = "ldlPcfb",
                     xlab = list(label = "LDL % change from baseline", cex = 1.2))
  print(update(ppcPlot, main = list(label = "Population Predictions", cex = 1.2)))
}     

################################################################################################
## random effect plots

effects <- c("e0")
effectsPost <- posterior[,unlist(sapply(c(paste("^",effects,"$",sep=""),
	paste("^",effects,"\\[",sep="")),grep,x=dimnames(posterior)[[2]]))]

x1 <- ldlData[!duplicated(ldlData$trial),]

medianEffects = sapply(effects,
  function(effect, posterior, n){
    eta <- t(posterior[,grep(paste(effect,"\\[",sep=""),dimnames(posterior)[[2]])])
    apply(eta,
          1, median)
  }, posterior = effectsPost)
colnames(medianEffects) <- effects
medianEffects <- cbind(x1, medianEffects)

for(effect in effects){

  print(histogram(~ medianEffects[effect], medianEffects,
            ylab = list(cex=1.2), xlab=list(label = effect, cex=1.2),
            scales=list(cex=1.2)))
  print(qqmath(~ medianEffects[effect], medianEffects,
         prepanel = prepanel.qqmathline, distribution = qnorm,
         panel=function(x, distribution){
           panel.qqmath(x,cex=1.2,pch=16, distribution = distribution)
           panel.qqmathline(x,distribution=distribution,col=3,lwd=3)
         },
         ylab=list(cex=1.2),
         xlab=list(label = effect, cex=1.2),
         scales=list(cex=1.2)))
  print(histogram(~ medianEffects[effect] | as.factor(drug1), medianEffects,
            ylab = list(cex=1.2), xlab=list(label = effect, cex=1.2),
            scales=list(cex=1.2)))
  print(qqmath(~ medianEffects[effect] | as.factor(drug1), medianEffects,
         prepanel = prepanel.qqmathline, distribution = qnorm,
         panel=function(x, distribution){
           panel.qqmath(x,cex=1.2,pch=16, distribution = distribution)
           panel.qqmathline(x,distribution=distribution,col=3,lwd=3)
         },
         ylab=list(label = effect, cex=1.2),
         xlab=list(cex=1.2),
         scales=list(cex=1.2)))
}

#########################################################################################
## residuals

## prediction of future observations in the same subjects, i.e., posterior predictions
## conditioned on observed data from the same subject

pred = posterior[,grep("pcldlCond\\[",dimnames(posterior)[[2]])]

pcldlPred <- apply(pred, 2, median, na.rm = T)

resid <- ldlData$ldlPcfb - pcldlPred

xyplot(resid ~ dose1, ldlData, panel =
       function(x, y, ...){
         panel.xyplot(jitter(x, 2), y, ...)
         panel.abline(h = 0, col = "red", lty = 2, lwd = 2)
       },
       cex=0.5,scales=list(cex=1),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="LDL % change from baseline residual", cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

xyplot(resid ~ pcldlPred, panel =
       function(x, y, ...){
         panel.xyplot(x, y, ...)
         panel.abline(h = 0, col = "red", lty = 2, lwd = 2)
       },
       cex=0.5,scales=list(cex=1),
       xlab=list(label="LDL % change from baseline predicted",cex=1.2),
          ylab=list(label="LDL % change from baseline residual", cex=1.2),
          par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

xyplot(resid ~ dose1 | as.factor(drug1),
       ldlData, panel =
       function(x, y, ...){
         panel.xyplot(jitter(x, 2), y, ...)
         panel.abline(h = 0, col = "red", lty = 2, lwd = 2)
       },
       cex=0.5,scales=list(cex=1),
       xlab=list(label="dose (mg/d)",cex=1.2),
       ylab=list(label="LDL % change from baseline residual", cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

xyplot(resid ~pcldlPred |
          as.factor(drug1), ldlData, panel =
       function(x, y, ...){
         panel.xyplot(x, y, ...)
         panel.abline(h = 0, col = "red", lty = 2, lwd = 2)
       },
       cex=0.5,scales=list(cex=1),
       xlab=list(label="LDL % change from baseline predicted",cex=1.2),
          ylab=list(label="LDL % change from baseline residual", cex=1.2),
          par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

histogram(~ resid,
       xlab=list(label="LDL % change from baseline residual",cex=1.2),
          ylab = list(cex=1.2),
          scales=list(cex=1.2))
qqmath(~ resid,
       prepanel = prepanel.qqmathline, distribution = qnorm,
       panel=function(x, distribution){
         panel.qqmath(x,cex=1.2,pch=16, distribution = distribution)
         panel.qqmathline(x,distribution=distribution,col=3,lwd=3)
       },
       ylab=list(label="LDL % change from baseline residual",cex=1.2),
       xlab=list(cex=1.2),
       scales=list(cex=1.2))

histogram(~ resid | as.factor(drug1), ldlData,
       xlab=list(label="LDL % change from baseline residual",cex=1.2),
          ylab = list(cex=1.2),
          scales=list(cex=1.2))
qqmath(~ resid | as.factor(drug1), ldlData,
       prepanel = prepanel.qqmathline, distribution = qnorm,
       panel=function(x, distribution){
         panel.qqmath(x,cex=1.2,pch=16, distribution = distribution)
         panel.qqmathline(x,distribution=distribution,col=3,lwd=3)
       },
       ylab=list(label="LDL % change from baseline residual",cex=1.2),
       xlab=list(cex=1.2),
       scales=list(cex=1.2))

dev.off()


