modelName <- "statinGemcabene" # root names of model file

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup

setwd(exampleDir)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

## Get data files
ldlData <- read.csv(file.path(exampleDir, "ldlData.csv"), as.is = TRUE)

## Rename 0 dose treatments as "Placebo"
ldlData$drug1 <- ifelse(ldlData$dose1 == 0, "Placebo", ldlData$drug1)
ldlData$drug2 <- ifelse(ldlData$dose2 == 0, "Placebo", ldlData$drug2)

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EDAPlots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

## Plots for statin monotherapy
drugs <- sort(unique(ldlData$drug1))
drugs <- drugs[drugs != "Placebo"]
xplot <- ldlData[ldlData$dose2 == 0,]
for(drug in drugs){
  trials <- unique(xplot$trial[xplot$drug1 == drug])
  yplot <- xplot[(xplot$trial %in% trials) & (xplot$drug1 %in% c(drug, "Placebo")),]

  print(xyplot(ldlPcfb ~ dose1 | as.factor(trial), yplot, groups = trial, panel =
               function(x, y, subscripts, ...){
                 panel.superpose(x, y, subscripts, ...)
                 panel.text(x, y + ifelse(x == 0, -5, 5),
                            labels = yplot$n[subscripts])
               },
               type = "b", lwd = 2, col = "blue", cex=1, scales=list(cex=1),
               xlab = list(label="dose (mg/d)", cex=1.2),
               ylab=list(label = "LDL change from baseline", cex=1.2),
               par.strip.text=list(cex=1),
               strip = function(...) strip.default(..., style = 1),
               main = list(label = drug, cex=1.5)))

}     

## Plots for non-statin monotherapy
drugs <- sort(unique(ldlData$drug2))
drugs <- drugs[drugs != "Placebo"]
xplot <- ldlData[ldlData$dose1 == 0,]
for(drug in drugs){
  trials <- unique(xplot$trial[xplot$drug2 == drug])
  yplot <- xplot[(xplot$trial %in% trials) & (xplot$drug2 %in% c(drug, "Placebo")),]

  print(xyplot(ldlPcfb ~ dose2 | as.factor(trial), yplot, groups = trial, panel =
               function(x, y, subscripts, ...){
                 panel.superpose(x, y, subscripts, ...)
                 panel.text(x, y + ifelse(x == 0, -5, 5),
                            labels = yplot$n[subscripts])
               },
               type = "b", lwd = 2, col = "blue", cex=1, scales=list(cex=1),
               xlab = list(label="dose (mg/d)", cex=1.2),
               ylab=list(label = "LDL change from baseline", cex=1.2),
               par.strip.text=list(cex=1),
               strip = function(...) strip.default(..., style = 1),
               main = list(label = drug, cex=1.5)))

}     


## Plots for statin/non-statin combinations

## Ezetimibe combinations
combTrials <- unique(ldlData$trial[ldlData$dose1 > 0 & ldlData$dose2 > 0 &
                                   ldlData$drug2 == "Ezetimibe"])
xplot <- ldlData[ldlData$trial %in% combTrials,]
drugs <- sort(unique(xplot$drug1))
drugs <- drugs[drugs != "Placebo"]
x1 <- xplot[xplot$drug1 == "Placebo",]
xplot <- xplot[xplot$drug1 != "Placebo",]
for(trial in combTrials){
  drug <- xplot$drug1[xplot$trial == trial][1]
  x2 <- x1[x1$trial == trial,]
  x2$drug1 <- rep(drug, nrow(x2))
  xplot <- rbind(xplot, x2)
}
xplot <- xplot[order(xplot$drug1, xplot$dose2, xplot$dose1),]
  
xyplot(ldlPcfb ~ dose1 | as.factor(drug1), xplot, groups = dose2, panel =
       function(x, y, subscripts, ...){
         panel.superpose(x, y, subscripts, ...)
         panel.text(x, y + ifelse(x == 0, -5, 5),
                    labels = xplot$n[subscripts])
       },
       type = "b", lwd = 2, col = c("blue", "red"), cex=1, scales=list(cex=1),
       xlab = list(label="dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL change from baseline", cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))


## gemcabene combinations
combTrials <- unique(ldlData$trial[ldlData$dose1 > 0 & ldlData$dose2 > 0 &
                                   ldlData$drug2 == "Gemcabene"])
xplot <- ldlData[ldlData$trial %in% combTrials,]

xyplot(ldlPcfb ~ dose1, xplot, groups = dose2, panel =
       function(x, y, subscripts, groups, ...){
         panel.superpose(x, y, subscripts, groups, ...)
         group <- as.numeric(as.factor(groups[subscripts]))
         panel.text(x + 2.5 * ((-1)^group), y + 2.5 * ((-1)^group),
                    labels = xplot$n[subscripts],
                    col = c("blue", "green", "orange", "red")[group])
       },
       type = "b", lwd = 2, col = c("blue", "green", "orange", "red"), cex=1, scales=list(cex=1),
       xlab = list(label="atorvastatin dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL change from baseline", cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1),
       key = list(x = 0.5, y = 0.75,
         lines = list(col = c("blue", "green", "orange", "red")),
         text = list(label = paste(sort(unique(xplot$dose2)), "mg/d")),
         title = "gemcabene dose", cex.title = 1.2))

dev.off()
