modelName <- "statinGemcabeneTruncEmax" # root names of model file
exampleName <- "statinGemcabeneSim"

## Logical indicating whether to do parallel processing on multiple cores
multiprocess <- TRUE

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI260W")
modelDir <- file.path(courseDir, modelName, modelName)
exampleDir <- file.path(courseDir, exampleName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
library(parallel)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))
source(file.path(courseDir, "mbmaPpcPlot.R"))

ldlPopMean <- function(p, design, interTrial = TRUE){
  ## Simulate population mean for each row of design
  
  ## Arguments
  ##   p = vector of model parameters, typically one row from a matrix of posterior samples
  ##   design = data frame containing independent variable values, e.g., dose, drug and any
  ##            other covariates required by the model
  ##   interTrial = logical indicating whether inter-trial variation is treated as a component
  ##                of the posterior uncertainty. Otherwise the population mean averages over
  ##                inter-trial variation

  p["nNonStatin[1]"] <- 1
  if(interTrial)
    e0 <- rnorm(1, p["e0Mean"], p["omegaE0"])
  else
    e0 <- p["e0Mean"] # because the model is linear w.r.t. this random effect
  eStatin <- p["emaxStatin"] * design$dose1^p["nStatin"] /
    (p[paste("ed50Statin[", design$drug1, "]", sep = "")]^p["nStatin"] + design$dose1^p["nStatin"])
  eNonStatin <- p[paste("emaxNonStatin[", design$drug2, "]", sep = "")] *
    design$dose2^p[paste("nNonStatin[", design$drug2, "]", sep = "")] /
    (p[paste("ed50NonStatin[", design$drug2, "]", sep = "")]^
     p[paste("nNonStatin[", design$drug2, "]", sep = "")] +
     design$dose2^p[paste("nNonStatin[", design$drug2, "]", sep = "")])
  ldlHat <- e0 - eStatin - eNonStatin + 0.01 * p[paste("gamma[", design$drug2, "]", sep = "")] *
    eStatin * eNonStatin
  names(ldlHat) <- NULL
  return(ldlHat)
}


## Specify parameters to keep
parametersToPlot <- c(
                      "e0Mean", "emaxStatin", "ed50Statin", "nStatin",
                      "emaxNonStatin", "ed50NonStatin", "nNonStatin",
                      "gamma", "omegaE0", "sigma",
                      "eStarStatin", "bStatin", "piStatin",
                      "eStarNonStatin", "bNonStatin"
                      )

################################################################################################

load(file.path(modelDir,paste(modelName,".fit.Rsave",sep="")))

## rename and reformat MCMC results to facilitate later calculations and plots
sims.array = bugs.fit$sims.array
posterior = array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
  dimnames=list(NULL,dimnames(sims.array)[[3]]))
posterior <- posterior[,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
	paste("^",parametersToPlot,"\\[",sep="")),grep,x=dimnames(posterior)[[2]]))]

################################################################################################

## Key for drug codes
## statins: atorvastatin = 1, rosuvastatin = 2, simvastatin = 3, lovastatin = 4, pravastatin = 5
## non-statins: ezetimibe = 1, gemcabene = 2

nSim <- 1000 # Number of posterior samples

## Set up design data frame
statinDoses <- c(0:4, seq(5, 80, by = 5))
ezetDoses <- c(0, 0.25, 0.5, 1, 2.5, 5, 10, 20, 30, 40)
gemDoses <- seq(0, 900, by = 50)
design <- rbind(expand.grid(drug1 = 1:5,
                      dose1 = statinDoses,
                      drug2 = 1,
                      dose2 = ezetDoses),
                expand.grid(drug1 = 1:5,
                      dose1 = statinDoses,
                      drug2 = 2,
                      dose2 = gemDoses))
## The above design data frame could be improved by removing redundant arms

## Simulate nSim posterior samples of the population mean for each row of design
#pred <- apply(posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
#      ldlPopMean, design = design, interTrial = FALSE)
if(multiprocess){
  cluster <- makeCluster(detectCores())
  clusterSetRNGStream(cl = cluster)
  pred <- parApply(cl = cluster, posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
                   ldlPopMean, design = design, interTrial = FALSE)
  stopCluster(cluster)
}else{
  pred <- apply(posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
                ldlPopMean, design = design, interTrial = FALSE)
}

## Example plots

dir.create(file.path(exampleDir, exampleName))
pdf(file = file.path(exampleDir, paste(exampleName,"/",exampleName,"Plots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

## Statin monotherapy dose-response

## Calculate median and 90% prediction interval
pred1 <- pred[design$drug2 == 1 & design$dose2 == 0,]
design1 <- design[design$drug2 == 1 & design$dose2 == 0,]
design1 <- rbind(design1, design1, design1)
design1$type = rep(c("5%ile","median","95%ile"), ea=nrow(pred1))
design1$pcldl <- as.vector(t(apply(pred1, 1, quantile, probs = c(0.05, 0.5, 0.95))))

## Replace drug1 with drug names
design1$drug1Name <- swap(design1$drug1, 1:5,
                          c("atorvastatin", "rosuvastatin", "simvastatin",
                            "lovastatin", "pravastatin"))

## Simple curves
xyplot(pcldl ~ dose1 | as.factor(drug1Name), design1, groups = type,
       type = "l", lwd = 2, col = c("blue", "blue", "red"), lty = c(2, 2, 1),
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

## Filled credible intervals
xyplot(pcldl ~ dose1 | as.factor(drug1Name), design1, groups = type, panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median")
         i5 <- (groups[subscripts] == "5%ile")
         i95 <- (groups[subscripts] == "95%ile")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
       },
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

## Non-statin monotherapy dose-response

## Calculate median and 90% prediction interval
pred1 <- pred[design$drug1 == 1 & design$dose1 == 0,]
design1 <- design[design$drug1 == 1 & design$dose1 == 0,]
design1 <- rbind(design1, design1, design1)
design1$type = rep(c("5%ile","median","95%ile"), ea=nrow(pred1))
design1$pcldl <- as.vector(t(apply(pred1, 1, quantile, probs = c(0.05, 0.5, 0.95))))

## Replace drug1 with drug names
design1$drug2Name <- swap(design1$drug2, 1:2,
                          c("ezetimibe", "gemcabene"))

xyplot(pcldl ~ dose2 | as.factor(drug2Name), design1, groups = type, panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median")
         i5 <- (groups[subscripts] == "5%ile")
         i95 <- (groups[subscripts] == "95%ile")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
       },
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2, x = list(relation = "free")),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

## Atorvastatin dose-response with and without ezetimibe 10 mg/d or gemcabene 900 mg/d

## Calculate median and 90% prediction interval
pred1 <- pred[design$drug1 == 1 &
              ((design$drug2 == 1 & design$dose2 %in% c(0, 10)) |
               (design$drug2 == 2 & design$dose2 %in% c(0, 900))),]
design1 <- design[design$drug1 == 1 &
              ((design$drug2 == 1 & design$dose2 %in% c(0, 10)) |
               (design$drug2 == 2 & design$dose2 %in% c(0, 900))),]
design1 <- rbind(design1, design1, design1)
design1$type = rep(c("5%ile","median","95%ile"), ea=nrow(pred1))
design1$pcldl <- as.vector(t(apply(pred1, 1, quantile, probs = c(0.05, 0.5, 0.95))))

## Replace drug1 with drug names
design1$drug2Name <- swap(design1$drug2, 1:2,
                          c("ezetimibe", "gemcabene"))

xyplot(pcldl ~ dose1 | as.factor(drug2Name), design1, groups = paste(type, dose2), panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median 0")
         i5 <- (groups[subscripts] == "5%ile 0")
         i95 <- (groups[subscripts] == "95%ile 0")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
         dose2 <- c(10, 900)[panel.number()]
         iMed <- (groups[subscripts] == paste("median", dose2))
         i5 <- (groups[subscripts] == paste("5%ile", dose2))
         i95 <- (groups[subscripts] == paste("95%ile", dose2))
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "red", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "red", ...)
       },
       xlab=list(label = "atorvastatin dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2, x = list(relation = "free")),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

################################################################################################

## Pr(LDL(ator+gem) > LDL(ator+ezet) | D(ator))

doses <- sort(unique(design1$dose1))
probBetter <- with(design1[design1$type == "median",],
                   sapply(doses,
                          function(doseStatin)
                          frac(pred1[dose1 == doseStatin & dose2 == 900,] <
                               pred1[dose1 == doseStatin & dose2 == 10,])))

xyplot(probBetter ~ doses,
       type = "l", col = "blue", lwd = 2,
       xlab=list(label = "atorvastatin dose (mg/d)", cex=1.2),
       ylab=list(label = "Pr(statin + gemcabene 900 mg/d superior to statin + ezetimibe 10 mg/d", cex=1),
       scales=list(cex=1.2))

## (LDL(ator+gem) - LDL(ator+ezet) | D(ator))

doses <- sort(unique(design1$dose1))
diff <- with(design1[design1$type == "median",],
                   sapply(doses,
                          function(doseStatin)
                          pred1[dose1 == doseStatin & dose2 == 900,] -
                               pred1[dose1 == doseStatin & dose2 == 10,]))

diff <- data.frame(dose1 = rep(doses, 3),
                   pcldlDiff = as.vector(t(apply(diff, 2, quantile, probs = c(0.05, 0.5, 0.95)))),
                   type = rep(c("5%ile","median","95%ile"), ea = length(doses)))

xyplot(pcldlDiff ~ dose1, diff, groups = type, panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median")
         i5 <- (groups[subscripts] == "5%ile")
         i95 <- (groups[subscripts] == "95%ile")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
         panel.abline(h = 0, col = "red", lwd = 2)
         panel.arrows(-2, -10, -2, 10, col = "red", lwd = 2, ends = "both")
         panel.text(c(5, 5), c(-7.5, 7.5),
                    c("statin + gemcabene better", "statin + ezetimibe better"),
                    adj = c(0,0), cex= 1.2)
       },
       xlab=list(label = "atorvastatin dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline\nDifference between statin + gemcabene 900 mg/d and statin + ezetimibe 10 mg/d", cex=1),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

## Distribution of D(ator) + gem equivalent to specified ator + ezet combos

statinDoses <- c(10, 20, 40, 80)
designMed <- design1[design1$type == "median",]
designRef <- with(designMed, designMed[dose1 %in% statinDoses & dose2 == 10,])
predRef <- with(designMed, pred1[dose1 %in% statinDoses & dose2 == 10,])
designTest <- with(designMed, designMed[dose2 == 900,])
predTest <- with(designMed, pred1[dose2 == 900,])

predRef <- predRef[order(designRef$dose1),]
designRef <- designRef[order(designRef$dose1),]
predTest <- predTest[order(designTest$dose1),]
designTest <- designTest[order(designTest$dose1),]

doseEquiv <- apply(predRef, 1,
                   function(yref, predTest, doseTest)
                   sapply(1:length(yref),
                          function(i, predTest, doseTest)
                          approx(predTest[,i], doseTest, yref[i], yleft = 100)$y,
                          predTest = predTest, doseTest = doseTest),
                   predTest = predTest, doseTest = designTest$dose1)
doseEquiv <- data.frame(doseRef = rep(designRef$dose1, ea = nrow(doseEquiv)),
                        doseTest = as.vector(doseEquiv))

histogram(~ doseTest | as.factor(paste("atorvastatin", doseRef, "mg/d\nezetimibe 10 mg/d")),
          doseEquiv, type = "density", nint = 25,
          xlab=list(label = "atorvastatin dose (mg/d) + gemcabene 900 mg/d\nequivalent to atorvastatin + ezetimibe 10 mg/d", cex=1.2),
          ylab=list(label = "probability density", cex=1.2),
          scales=list(cex=1.2,
            x = list(at = seq(0, 100, by = 20), labels = c(seq(0, 80, by = 20), ">80")),
              y = list(relation = "free")),
          par.strip.text=list(cex=1, lines = 2),
          strip = function(...) strip.default(..., style = 1))

dev.off()
