modelName <- "statinReparHandsOn" # root names of model file
exampleName <- "statinSimHandsOn"

## Logical indicating whether to do parallel processing on multiple cores
multiprocess <- TRUE

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
courseDir <- file.path(Sys.getenv("HOME"), "project/MI260W")
modelDir <- file.path(courseDir, modelName, modelName)
exampleDir <- file.path(courseDir, exampleName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
library(parallel)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

statinPopMean <- function(p, design, interTrial = TRUE){
  ## Simulate population mean for each row of design
  
  ## Arguments
  ##   p = vector of model parameters, typically one row from a matrix of posterior samples
  ##   design = data frame containing independent variable values, e.g., dose, drug and any
  ##            other covariates required by the model
  ##   interTrial = logical indicating whether inter-trial variation is treated as a component
  ##                of the posterior uncertainty. Otherwise the population mean averages over
  ##                inter-trial variation

  p["nNonStatin[1]"] <- 1
  if(interTrial)
    e0 <- rnorm(1, p["e0Mean"], p["omegaE0"])
  else
    e0 <- p["e0Mean"] # because the model is linear w.r.t. this random effect
  eStatin <- p["emaxStatin"] * design$dose^p["nStatin"] /
    (p[paste("ed50Statin[", design$drug, "]", sep = "")]^p["nStatin"] + design$dose^p["nStatin"])
  ldlHat <- e0 - eStatin
  names(ldlHat) <- NULL
  return(ldlHat)
}

## Specify parameters to keep
parametersToPlot <- c(
                      "e0Mean", "emaxStatin", "ed50Statin", "nStatin",
                      "omegaE0", "sigma"
                      )

################################################################################################

load(file.path(modelDir,paste(modelName,".fit.Rsave",sep="")))

## rename and reformat MCMC results to facilitate later calculations and plots
sims.array = bugs.fit$sims.array
posterior = array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
  dimnames=list(NULL,dimnames(sims.array)[[3]]))
posterior <- posterior[,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
	paste("^",parametersToPlot,"\\[",sep="")),grep,x=dimnames(posterior)[[2]]))]

################################################################################################

dir.create(file.path(exampleDir, exampleName))
pdf(file = file.path(exampleDir, paste(exampleName,"/",exampleName,"Plots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

## Key for drug codes
## statins: atorvastatin = 1, rosuvastatin = 2

nSim <- 1000 # Number of posterior samples

## Set up design data frame
statinDoses <- c(0:4, seq(5, 80, by = 5))
design <- expand.grid(drug = 1:2,
                      dose = statinDoses)

## Statin dose-response -- averaging over inter-trial variation

## Simulate nSim posterior samples of the population mean for each row of design
if(multiprocess){
  cluster <- makeCluster(detectCores())
  clusterSetRNGStream(cl = cluster)
  pred <- parApply(cl = cluster, posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
                   statinPopMean, design = design, interTrial = FALSE)
  stopCluster(cluster)
}else{
  pred <- apply(posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
                statinPopMean, design = design, interTrial = FALSE)
}

## Calculate median and 90% prediction interval
design1 <- rbind(design, design, design)
design1$type = rep(c("5%ile","median","95%ile"), ea=nrow(pred))
design1$pcldl <- as.vector(t(apply(pred, 1, quantile, probs = c(0.05, 0.5, 0.95))))

## Replace drug with drug names
design1$drugName <- swap(design1$drug, 1:2,
                         c("atorvastatin", "rosuvastatin"))

## Simple curves
xyplot(pcldl ~ dose | as.factor(drugName), design1, groups = type,
       type = "l", lwd = 2, col = c("blue", "blue", "red"), lty = c(2, 2, 1),
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1),
       sub = "averaged over inter-trial variation")

## Filled credible intervals
xyplot(pcldl ~ dose | as.factor(drugName), design1, groups = type, panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median")
         i5 <- (groups[subscripts] == "5%ile")
         i95 <- (groups[subscripts] == "95%ile")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
       },
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1),
       sub = "averaged over inter-trial variation")

## Statin dose-response -- inter-trial variation included in uncertainty bounds

## Simulate nSim posterior samples of the population mean for each row of design
if(multiprocess){
  cluster <- makeCluster(detectCores())
  clusterSetRNGStream(cl = cluster)
  pred <- parApply(cl = cluster, posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
                   statinPopMean, design = design, interTrial = TRUE)
  stopCluster(cluster)
}else{
  pred <- apply(posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
                statinPopMean, design = design, interTrial = TRUE)
}

## Calculate median and 90% prediction interval
design1 <- rbind(design, design, design)
design1$type = rep(c("5%ile","median","95%ile"), ea=nrow(pred))
design1$pcldl <- as.vector(t(apply(pred, 1, quantile, probs = c(0.05, 0.5, 0.95))))

## Replace drug with drug names
design1$drugName <- swap(design1$drug, 1:2,
                         c("atorvastatin", "rosuvastatin"))

## Filled credible intervals
xyplot(pcldl ~ dose | as.factor(drugName), design1, groups = type, panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median")
         i5 <- (groups[subscripts] == "5%ile")
         i95 <- (groups[subscripts] == "95%ile")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
       },
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1),
       sub = "inter-trial variation included in uncertainty")

################################################################################################

## Pr(LDL(rosu) > LDL(ator) | D(rosu))

nSim <- 1000 # Number of posterior samples

## Set up design data frame
statinDoses <- 0:80
design <- expand.grid(drug = 1:2,
                      dose = statinDoses)

## Statin dose-response -- averaging over inter-trial variation

## Simulate nSim posterior samples of the population mean for each row of design
if(multiprocess){
  cluster <- makeCluster(detectCores())
  clusterSetRNGStream(cl = cluster)
  pred <- parApply(cl = cluster, posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
                   statinPopMean, design = design, interTrial = FALSE)
  stopCluster(cluster)
}else{
  pred <- apply(posterior[sample(1:nrow(posterior), nSim, replace = (nSim > nrow(posterior))),], 1,
                statinPopMean, design = design, interTrial = FALSE)
}

doses <- sort(unique(design$dose))
doseAtor <- c(10, 20, 40, 80)
probBetter <- with(design,
                   sapply(doseAtor,
                          function(doseAtor, doses)
                          sapply(doses,
                                 function(doseRosu, doseAtor)
                                 frac(pred[drug == 2 & dose == doseRosu,] <
                                      pred[drug == 1 & dose == doseAtor,]),
                                 doseAtor = doseAtor),
                          doses = doses))
probBetter <- data.frame(doseRosu = rep(doses, length(doseAtor)),
                         doseAtor = rep(doseAtor, ea = length(doses)),
                         probBetter = as.vector(probBetter))

xyplot(probBetter ~ doseRosu | as.factor(paste("atorvastatin", doseAtor, "mg/d")), probBetter,
       type = "l", col = "blue", lwd = 2,
       xlab=list(label = "rosuvastatin dose (mg/d)", cex=1.2),
       ylab=list(label = "Pr(rosuvastatin superior to atorvastatin", cex=1),
       scales=list(cex=1.2))

## (LDL(rosu) - LDL(ator) | D(rosu))

doses <- sort(unique(design$dose))
doseAtor <- c(10, 20, 40, 80)
diff <- with(design,
                   sapply(doseAtor,
                          function(doseAtor, doses)
                          sapply(doses,
                                 function(doseRosu, doseAtor)
                                 quantile(pred[drug == 2 & dose == doseRosu,] -
                                      pred[drug == 1 & dose == doseAtor,], probs = c(0.05, 0.5, 0.95)),
                                 doseAtor = doseAtor),
                          doses = doses))
diff <- data.frame(doseRosu = rep(rep(doses, ea = 3), length(doseAtor)),
                   doseAtor = rep(doseAtor, ea = 3 * length(doses)),
                   pcldlDiff = as.vector(diff),
                   type = rep(c("5%ile","median","95%ile"), length(doseAtor) * length(doses)))

xyplot(pcldlDiff ~ doseRosu | as.factor(paste("atorvastatin", doseAtor, "mg/d")), diff, groups = type, panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median")
         i5 <- (groups[subscripts] == "5%ile")
         i95 <- (groups[subscripts] == "95%ile")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
         panel.abline(h = 0, col = "red", lwd = 2)
         panel.arrows(-2, -20, -2, 20, col = "red", lwd = 2, ends = "both")
         panel.text(c(5, 5), c(-15, 15),
                    c("rosuvastatin better", "atorvastin better"),
                    adj = c(0,0), cex= 1.2)
       },
       xlab=list(label = "rosuvastatin dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline\nDifference between rosuvastatin and atorvastatin", cex=1),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1))

## Distribution of D(rosu) equivalent to specified ator doses

## Construct separate ator and rosu pred and design objects
doseAtor <- c(10, 20, 40, 80)
designAtor <- with(design, design[dose %in% doseAtor & drug == 1,])
predAtor <- with(design, pred[dose %in% doseAtor & drug == 1,])
designRosu <- with(design, design[drug == 2,])
predRosu <- with(design, pred[drug == 2,])

## Order by dose to facilitate interpolation
predAtor <- predAtor[order(designAtor$dose),]
designAtor <- designAtor[order(designAtor$dose),]
predRosu <- predRosu[order(designRosu$dose),]
designRosu <- designRosu[order(designRosu$dose),]

doseEquiv <- apply(predAtor, 1,
                   function(yref, predRosu, doseRosu)
                   sapply(1:length(yref),
                          function(i, predRosu, doseRosu)
                          approx(predRosu[,i], doseRosu, yref[i], yleft = 100)$y,
                          predRosu = predRosu, doseRosu = doseRosu),
                   predRosu = predRosu, doseRosu = designRosu$dose)
doseEquiv <- data.frame(doseAtor = rep(designAtor$dose, ea = nrow(doseEquiv)),
                        doseRosu = as.vector(doseEquiv))

histogram(~ doseRosu | as.factor(paste("atorvastatin", doseAtor, "mg/d")),
          doseEquiv, type = "density", nint = 25,
          xlab=list(label = "rosuvastatin dose (mg/d) equivalent to atorvastatin", cex=1.2),
          ylab=list(label = "probability density", cex=1.2),
          scales=list(cex=1.2,
            y = list(relation = "free")),
          par.strip.text=list(cex=1),
          strip = function(...) strip.default(..., style = 1))

dev.off()
